------------------------------------------------------------------------------
-- Trilby HAT - sample firmware
--
-- Copyright (c) 2005-2016 Kinetic Avionics Ltd
-- www.kinetic.co.uk
--
-- Permission is hereby granted, free of charge, to any person obtaining a
-- copy of this software and associated documentation files (the "Software"),
-- to deal in the Software without restriction, including without limitation
-- the rights to use, copy, modify, merge, publish, distribute, sublicense,
-- and/or sell copies of the Software, and to permit persons to whom the
-- Software is furnished to do so, subject to the following conditions:
-- The above copyright notice and this permission notice shall be included
-- in all copies or substantial portions of the Software.
-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
-- EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-- MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-- IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
-- DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
-- TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
-- OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
-- 
-- Module Name: adc_sync
-- Purpose: Provides I/Q sample generation from ADC inputs
-- Author: Keith Frewin
--
-- Version History:
-- dd/mm/yyyy Comments
-- 14/04/2016 Initial release
------------------------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

entity adc_sync is
    port (
        fastclk: in STD_LOGIC;
        adcclk: in STD_LOGIC;
       adcin: IN STD_LOGIC_VECTOR (13 DOWNTO 0);
       sclk:out STD_LOGIC;
		iout: out STD_LOGIC_VECTOR (17 DOWNTO 0);
		qout: out STD_LOGIC_VECTOR (17 DOWNTO 0)
    );
end adc_sync;


architecture rtl of adc_sync is

constant DATA_WIDTH: integer := 18;

signal lastadcclk: STD_LOGIC;
signal deladcclk: STD_LOGIC;
signal ADCInReg: signed (13 DOWNTO 0);
signal ClkOutReg: STD_LOGIC;

signal MixCount: unsigned (1 downto 0);
signal Inreg: signed (DATA_WIDTH-1 downto 0);
signal QInreg: signed (DATA_WIDTH-1 downto 0);
signal IOutreg: signed (DATA_WIDTH-1 downto 0);
signal QOutreg: signed (DATA_WIDTH-1 downto 0);


begin --arch
  
Inreg <= signed(ADCInReg(13 downto 2) & "000000");
iout <= STD_LOGIC_VECTOR(IOutreg);
qout <= STD_LOGIC_VECTOR(QOutreg);
sclk <= ClkOutReg;


process(adcclk)  -- adc latch 	
begin
	-- capture ADC data on falling edge of clock
	if falling_edge(adcclk) then
		ADCInReg <= signed (adcin);
   end if;
end process;

process(fastclk)  -- generate start pulse	
begin
	if rising_edge(fastclk) then
  
	  	if deladcclk = '0' and lastadcclk = '1' then	
			case MixCount is
			when "00" =>
				IOutReg <= Inreg;
				QOutReg <= (others => '0'); 
			when "01" =>
				IOutReg <= (others => '0');
				QOutReg <= -Inreg; 
			when "10" =>
				IOutReg <= -Inreg;
				QOutReg <= (others => '0'); 
			when "11" =>
				IOutReg <= (others => '0');
				QOutReg <= Inreg; 
			when others =>
			end case;
			MixCount <= MixCount + 1;
			ClkOutReg <= '1';
		else
			ClkOutReg <= '0';
		end if;
		deladcclk <= adcclk;
		lastadcclk <= deladcclk; 
	end if;
end process;
  
  


end rtl;
