------------------------------------------------------------------------------
-- Trilby HAT - sample firmware
--
-- Copyright (c) 2005-2016 Kinetic Avionics Ltd
-- www.kinetic.co.uk
--
-- Permission is hereby granted, free of charge, to any person obtaining a
-- copy of this software and associated documentation files (the "Software"),
-- to deal in the Software without restriction, including without limitation
-- the rights to use, copy, modify, merge, publish, distribute, sublicense,
-- and/or sell copies of the Software, and to permit persons to whom the
-- Software is furnished to do so, subject to the following conditions:
-- The above copyright notice and this permission notice shall be included
-- in all copies or substantial portions of the Software.
-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
-- EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-- MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-- IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
-- DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
-- TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
-- OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
-- 
-- Module Name: audio_pwm
-- Purpose: Generates PWM waveforms from digital audio samples
-- Author: Keith Frewin
--
-- Version History:
-- dd/mm/yyyy Comments
-- 14/04/2016 Initial release
------------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

entity audio_pwm is
    port (
		clk: in STD_LOGIC;
		res: in STD_LOGIC;
		sclk: in STD_LOGIC;
		data_in: in STD_LOGIC_VECTOR (17 downto 0);
		pwm_out: out STD_LOGIC
    );
end  audio_pwm;

architecture rtl of  audio_pwm is

constant CLK_FREQ: integer := 96000000;
constant PWM_FREQ: integer := 48000;
 
signal DutyReg: STD_LOGIC_VECTOR(10 downto 0);
signal PWMCount: unsigned(10 downto 0);
signal DecCount: unsigned(7 downto 0);

begin --arch
  

process (clk, res)
begin
	if res = '0' then
		PWMCount <= (others => '0');
		pwm_out <= '0';
		DecCount <= (others => '0');
		DutyReg <= (others => '0');
	elsif rising_edge(clk) then
		if (sclk = '1' )  then
			if DecCount >= 19 then
				DecCount <= (others => '0');
				DutyReg <= (not data_in(17)) & data_in(16 downto 7);	-- 11 bits gives 2048 steps
				PWMCount <= (others => '0');
			else
				PWMCount <= PWMCount + 1;  
				DecCount <= DecCount + 1;
			end if;
		else
			PWMCount <= PWMCount + 1;  -- counts from 0 to 1999 (96 MHz / 48 KHz)
		end if;
		if PWMCount < unsigned(DutyReg) then
			pwm_out <= '1';
		else
			pwm_out <= '0';
		end if;
			
	end if;
end process;

end rtl;





