------------------------------------------------------------------------------
-- Trilby HAT - sample firmware
--
-- Copyright (c) 2005-2016 Kinetic Avionics Ltd
-- www.kinetic.co.uk
--
-- Permission is hereby granted, free of charge, to any person obtaining a
-- copy of this software and associated documentation files (the "Software"),
-- to deal in the Software without restriction, including without limitation
-- the rights to use, copy, modify, merge, publish, distribute, sublicense,
-- and/or sell copies of the Software, and to permit persons to whom the
-- Software is furnished to do so, subject to the following conditions:
-- The above copyright notice and this permission notice shall be included
-- in all copies or substantial portions of the Software.
-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
-- EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-- MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-- IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
-- DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
-- TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
-- OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
-- 
-- Module Name: io_mux
-- Purpose: Multiplexes the command messages to/from the various interfaces
--          (SPI, I2C and serial)
-- Author: Keith Frewin
--
-- Version History:
-- dd/mm/yyyy Comments
-- 14/04/2016 Initial release
------------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

entity io_mux is
    port (
		rd_in_0: in STD_LOGIC;
		rd_in_1: in STD_LOGIC;
		rd_in_2: in STD_LOGIC;
		wr_in_0: in STD_LOGIC;
		wr_in_1: in STD_LOGIC;
		wr_in_2: in STD_LOGIC;
		addr_in_0: in STD_LOGIC_VECTOR (15 DOWNTO 0);
		addr_in_1: in STD_LOGIC_VECTOR (15 DOWNTO 0);
		addr_in_2: in STD_LOGIC_VECTOR (15 DOWNTO 0);
		data_in_0: in STD_LOGIC_VECTOR (7 DOWNTO 0);
		data_in_1: in STD_LOGIC_VECTOR (7 DOWNTO 0);
		data_in_2: in STD_LOGIC_VECTOR (7 DOWNTO 0);
		rd_out: out STD_LOGIC;
		wr_out: out STD_LOGIC;
		addr_out: out STD_LOGIC_VECTOR (15 DOWNTO 0);
		data_out: out STD_LOGIC_VECTOR (7 DOWNTO 0)
    );
end io_mux;

architecture rtl of io_mux is


begin --architecture
  
addr_out <= addr_in_0 when (wr_in_0 = '1') or (rd_in_0 = '1')
	else addr_in_1 when (wr_in_1 = '1') or (rd_in_1 = '1')
	else addr_in_2;
data_out <= data_in_0 when wr_in_0 = '1'
	else data_in_1 when wr_in_1 = '1'
	else data_in_2;
wr_out <= '1' when (wr_in_0 = '1') or (wr_in_1 = '1') or (wr_in_2 = '1') else '0';
rd_out <= '1' when (rd_in_0 = '1') or (rd_in_1 = '1') or (rd_in_2 = '1') else '0';
	

end rtl;





