------------------------------------------------------------------------------
-- Trilby HAT - sample firmware
--
-- Copyright (c) 2005-2016 Kinetic Avionics Ltd
-- www.kinetic.co.uk
--
-- Permission is hereby granted, free of charge, to any person obtaining a
-- copy of this software and associated documentation files (the "Software"),
-- to deal in the Software without restriction, including without limitation
-- the rights to use, copy, modify, merge, publish, distribute, sublicense,
-- and/or sell copies of the Software, and to permit persons to whom the
-- Software is furnished to do so, subject to the following conditions:
-- The above copyright notice and this permission notice shall be included
-- in all copies or substantial portions of the Software.
-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
-- EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-- MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-- IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
-- DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
-- TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
-- OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
-- 
-- Module Name: mix_iq
-- Purpose: Mixer with I and Q channels
-- Author: Keith Frewin
--
-- Version History:
-- dd/mm/yyyy Comments
-- 14/04/2016 Initial release
------------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

entity mix_iq is
    port (
		clk: in STD_LOGIC;
		res: in STD_LOGIC;
		sclkin: in STD_LOGIC;
		idatain: in STD_LOGIC_VECTOR (17 DOWNTO 0);
		qdatain: in STD_LOGIC_VECTOR (17 DOWNTO 0);
		sclkout: out STD_LOGIC;
		iout: out STD_LOGIC_VECTOR (17 DOWNTO 0);
		qout: out STD_LOGIC_VECTOR (17 DOWNTO 0)
    );
end mix_iq;

architecture rtl of mix_iq is

constant DATA_WIDTH: integer := 18;

signal MixCount: unsigned (1 downto 0);
signal IInreg: signed (DATA_WIDTH-1 downto 0);
signal QInreg: signed (DATA_WIDTH-1 downto 0);
signal IOutreg: signed (DATA_WIDTH-1 downto 0);
signal QOutreg: signed (DATA_WIDTH-1 downto 0);


begin --arch
  
iout <= STD_LOGIC_VECTOR(IOutreg);
qout <= STD_LOGIC_VECTOR(QOutreg);

process (clk, res)
begin
	if res = '0' then
		MixCount <= (others => '0');
		IInreg <= (others => '0');
		QInreg <= (others => '0');
		IOutreg <= (others => '0');
		QOutreg <= (others => '0');
		sclkout <= '0';
	elsif rising_edge(clk) then
		if sclkin = '1' then
			-- new input sample is ready
			case MixCount is
			when "00" =>
				IOutReg <= IInreg;
				QOutReg <= QInreg; 
			when "01" =>
				IOutReg <= QInreg;
				QOutReg <= -IInreg; 
			when "10" =>
				IOutReg <= -IInreg;
				QOutReg <= -QInreg; 
			when "11" =>
				IOutReg <= -QInreg;
				QOutReg <= IInreg; 
			when others =>
			end case;
			MixCount <= MixCount + 1;
			IInReg <= Signed(idatain);
			QInReg <= Signed(qdatain);
			sclkout <= '1';
		else
			sclkout <= '0';
		end if;

	end if;


end process;

end rtl;





